@@ -53,41 +53,125 @@ ESExpResult *func_debug(ESExp *f, int argc, ESExpResult **argv, Context *c) {
 }
 
 /**
- * Print args
+ * Transform parameters into a string and concat them
  */
-ESExpResult *func_print(ESExp *f, int argc, ESExpResult **argv, Context *c) {
-  int i;
-  for (i = 0; i < argc; i++) {
-    switch (argv[i]->type) {
-    case ESEXP_RES_ARRAY_PTR:
-      g_print (_("(array pointer: %p)\n"), argv[i]->value.ptrarray);
-      break;
-    case ESEXP_RES_BOOL:
-      g_print (argv[i]->value.bool ? _("TRUE\n") : _("FALSE\n"));
-      break;
-    case ESEXP_RES_INT:
-      g_print ("%d\n", argv[i]->value.number);
-      break;
-    case ESEXP_RES_TIME:
-      {
-        char buf[256];
-        struct tm time;
-        localtime_r(&argv[i]->value.time, &time);
-        strftime(buf, sizeof(buf), "%c", &time);
-        g_print ("time: %s", buf);
-        break;
-      }
-    case ESEXP_RES_STRING:
-      g_print ("%s\n", argv[i]->value.string);
-      break;
-    case ESEXP_RES_UNDEFINED:
-      g_print (_("(undefined)"));
+static char * internal_str(ESExp *f, int argc, ESExpResult **argv,
+			   Context *c)
+{
+  char *s_arg0, *s_other_args, *result = NULL;
+
+  if (argc < 1)
+    return NULL;
+
+  switch (argv[0]->type) {
+  case ESEXP_RES_BOOL:
+    s_arg0 = g_strdup((argv[0]->value.bool ? "TRUE" : "FALSE"));
+    break;
+
+  case ESEXP_RES_INT:
+    s_arg0 = g_strdup_printf("%d", argv[0]->value.number);
+    break;
+
+  case ESEXP_RES_TIME:
+    {
+      char buf[256];
+      struct tm time;
+      localtime_r(&argv[0]->value.time, &time);
+      strftime(buf, sizeof(buf), "%c", &time);
+      s_arg0 = g_strdup(buf);
       break;
     }
+
+  case ESEXP_RES_STRING:
+    s_arg0 = g_strdup(argv[0]->value.string);
+    break;
+
+  case ESEXP_RES_ARRAY_PTR:
+    s_arg0 = g_strdup_printf(_("(array pointer: %p)"),
+			     argv[0]->value.ptrarray);
+    break;
+
+  case ESEXP_RES_UNDEFINED:
+  default:
+    g_printerr(_("Cannot convert element into a string"));
+    return NULL;
+    break;
   }
+
+  s_other_args = internal_str(f, argc-1, argv+1, c);
+
+  if (s_arg0 && s_other_args)
+    {
+      result = g_strdup_printf("%s%s", s_arg0, s_other_args);
+      g_free(s_arg0);
+      g_free(s_other_args);
+    }
+  else if (s_arg0)
+    result = s_arg0;
+  else if (s_other_args)
+    result = s_other_args;
+
+  return result;
+}
+
+/**
+ * Print args (without trailing \n).
+ */
+ESExpResult *func_print(ESExp *f, int argc, ESExpResult **argv, Context *c) {
+  char * s = internal_str(f, argc, argv, c);
+  if (! s)
+    return e_sexp_result_new_bool (f, FALSE);
+
+  g_print("%s", s);
+  g_free(s);
+
   return e_sexp_result_new_bool (f, TRUE);
 }
 
+/**
+ * Print args (with trailing \n).
+ */
+ESExpResult *func_println(ESExp *f, int argc, ESExpResult **argv, Context *c) {
+  char * s = internal_str(f, argc, argv, c);
+  if (! s)
+    return e_sexp_result_new_bool (f, FALSE);
+
+  g_print("%s\n", s);
+  g_free(s);
+
+  return e_sexp_result_new_bool (f, TRUE);
+}
+
+/**
+ * Transform parameters into a string and concat them.
+ */
+ESExpResult *func_str(ESExp *f, int argc, ESExpResult **argv, Context *c)
+{
+  ESExpResult *r;
+  r = e_sexp_result_new(f, ESEXP_RES_STRING);
+  r->value.string = internal_str(f, argc, argv, c);
+  if (! r->value.string)
+    r->value.string = g_strdup("");
+  return r;
+}
+
+/**
+ * Transform the integer parameter into an unsigned hexadecimal string
+ * (with 0x prefix).
+ */
+ESExpResult *func_hex(ESExp *f, int argc, ESExpResult **argv, Context *c)
+{
+  ESExpResult *r;
+
+  if (argc != 1 || argv[0]->type != ESEXP_RES_INT)
+    return e_sexp_result_new_bool (f, FALSE); 
+
+  r = e_sexp_result_new(f, ESEXP_RES_STRING);
+  r->value.string = g_strdup_printf("0x%x", (unsigned)argv[0]->value.number);
+  return r;
+}
+
+
 /**
  * Set position + size of current window.
  */
@@ -310,7 +394,7 @@ ESExpResult *func_unpin(ESExp *f, int argc, ESExpResult **argv, Context *c) {
 }
 
 /**
- * Make the current window stick to all viewports
+ * Make the current window stick to all viewports.
  */
 ESExpResult *func_stick(ESExp *f, int argc, ESExpResult **argv, Context *c) {
   wnck_window_stick (c->window);
@@ -319,7 +403,7 @@ ESExpResult *func_stick(ESExp *f, int argc, ESExpResult **argv, Context *c) {
 }
 
 /**
- * Unstick the window from viewports
+ * Unstick the window from viewports.
  */
 ESExpResult *func_unstick(ESExp *f, int argc, ESExpResult **argv, Context *c) {
   wnck_window_unstick (c->window);
@@ -355,7 +439,7 @@ ESExpResult *func_set_workspace(ESExp *f, int argc, ESExpResult **argv, Context
 }
 
 /**
- * Move the window to a specific viewport number, counting from 1
+ * Move the window to a specific viewport number, counting from 1.
  */
 ESExpResult *func_set_viewport(ESExp *f, int argc, ESExpResult **argv, Context *c) {
   WnckScreen *screen;
@@ -525,6 +609,10 @@ ESExpResult *func_wintype(ESExp *f, int argc, ESExpResult **argv, Context *c) {
   return e_sexp_result_new_bool (f, TRUE);
 }
 
+/**
+ * Change the opacity level (as integer in 0..100) of the current
+ * window (returns boolean).
+ */
 ESExpResult *func_opacity(ESExp *f, int argc, ESExpResult **argv, Context *c) {
 	int opacity;
 	unsigned int v;
@@ -547,3 +635,136 @@ ESExpResult *func_opacity(ESExp *f, int argc, ESExpResult **argv, Context *c) {
 	my_wnck_error_trap_pop ();
 	return e_sexp_result_new_bool (f, TRUE);
 }
+
+/**
+ * Execute a command in the foreground (returns command output as
+ * string, or FALSE on error). Command is given as a single string, or
+ * as a series of strings (similar to execl).
+ */
+ESExpResult *func_spawn_sync(ESExp *f, int argc, ESExpResult **argv, Context *c)
+{
+  gboolean spawn_result;
+  gint exit_status = 0;
+  gchar * stdout_contents = NULL;
+  GError * errs = NULL;
+  ESExpResult *r;
+
+  if (argc < 1)
+    return e_sexp_result_new_bool (f, FALSE);
+
+  if (argc == 1) {
+    if (debug) g_printerr(_("spawn_sync(string)\n"));
+    if (argv[0]->type != ESEXP_RES_STRING) {
+      g_printerr(_("spawn_sync: Command parameter is not a string"));
+      return e_sexp_result_new_bool (f, FALSE);
+    }
+    
+    spawn_result = g_spawn_command_line_sync(argv[0]->value.string,
+					     & stdout_contents, NULL,
+					     & exit_status, & errs);
+  }
+  else {
+    /* Build spawn arguments */
+    gchar ** spawn_args = g_new(char*, argc+1);
+    int i;
+
+    if (debug) g_printerr(_("spawn_sync(list)\n"));
+
+    if (! spawn_args)
+      return e_sexp_result_new_bool (f, FALSE);
+
+    for (i = 0 ; i < argc ; i ++) {
+      if (argv[i]->type != ESEXP_RES_STRING) {
+	g_printerr(_("spawn_sync: Command parameter %d is not a string"), i);
+	g_free(spawn_args);
+	return e_sexp_result_new_bool (f, FALSE);
+      }
+      
+      spawn_args[i] = argv[i]->value.string;
+    }
+    spawn_args[i] = NULL;
+
+    spawn_result = g_spawn_sync(NULL, spawn_args, NULL, G_SPAWN_SEARCH_PATH,
+				NULL, NULL, &stdout_contents, NULL,
+				& exit_status, & errs);
+    g_free(spawn_args);
+  }
+
+  if (! spawn_result) {
+    if (errs && errs->message)
+      g_printerr(_("Error spawning child process (fg): %s\n"), errs->message);
+    else
+      g_printerr(_("Error spawning child process (fg).\n"));
+  }
+  if (exit_status)
+    g_printerr(_("Warning: child process returned %d\n"), exit_status);
+
+  if (! stdout_contents)
+      return e_sexp_result_new_bool (f, spawn_result);
+
+  r = e_sexp_result_new(f, ESEXP_RES_STRING);
+  r->value.string = stdout_contents;
+  return r;
+}
+
+/**
+ * Execute a command in the background (returns boolean). Command is
+ * given as a single string, or as a series of strings (similar to
+ * execl).
+ */
+ESExpResult *func_spawn_async(ESExp *f, int argc, ESExpResult **argv, Context *c)
+{
+  gboolean spawn_result;
+  GError * errs = NULL;
+
+  if (argc < 1)
+    return e_sexp_result_new_bool (f, FALSE);
+
+  if (argc == 1) {
+    if (debug) g_printerr(_("spawn_async(string)\n"));
+    if (argv[0]->type != ESEXP_RES_STRING) {
+      g_printerr(_("spawn_async: Command parameter is not a string"));
+      return e_sexp_result_new_bool (f, FALSE);
+    }
+    
+    spawn_result = g_spawn_command_line_async(argv[0]->value.string, & errs);
+  }
+  else {
+    /* Build spawn arguments */
+    gchar ** spawn_args = g_new(char*, argc+1);
+    int i;
+    GPid child_pid;
+
+    if (debug) g_printerr(_("spawn_async(list)\n"));
+
+    if (! spawn_args)
+      return e_sexp_result_new_bool (f, FALSE);
+
+    for (i = 0 ; i < argc ; i ++) {
+      if (argv[i]->type != ESEXP_RES_STRING) {
+	g_printerr(_("spawn_async: Command parameter %d is not a string"), i);
+	g_free(spawn_args);
+	return e_sexp_result_new_bool (f, FALSE);
+      }
+      
+      spawn_args[i] = argv[i]->value.string;
+    }
+    spawn_args[i] = NULL;
+
+    spawn_result = g_spawn_async(NULL, spawn_args, NULL, G_SPAWN_SEARCH_PATH,
+				 NULL, NULL, &child_pid, & errs);
+    g_free(spawn_args);
+
+    if (spawn_result && debug) g_printerr(_("Spawned pid %u (bg)\n"),
+					  child_pid);
+  }
+
+  if (! spawn_result) {
+    if (errs && errs->message)
+      g_printerr(_("Error spawning child process (bg): %s\n"), errs->message);
+    else
+      g_printerr(_("Error spawning child process (bg).\n"));
+  }
+  
+  return e_sexp_result_new_bool (f, spawn_result);
+}
