@@ -23,6 +23,9 @@
 
 ESExpResult *func_debug(ESExp *f, int argc, ESExpResult **argv, Context *c);
 ESExpResult *func_print(ESExp *f, int argc, ESExpResult **argv, Context *c);
+ESExpResult *func_println(ESExp *f, int argc, ESExpResult **argv, Context *c);
+ESExpResult *func_str(ESExp *f, int argc, ESExpResult **argv, Context *c);
+ESExpResult *func_hex(ESExp *f, int argc, ESExpResult **argv, Context *c);
 
 ESExpResult *func_geometry(ESExp *f, int argc, ESExpResult **argv, Context *c);
 ESExpResult *func_fullscreen(ESExp *f, int argc, ESExpResult **argv, Context *c);
@@ -57,4 +60,7 @@ ESExpResult *func_wintype(ESExp *f, int argc, ESExpResult **argv, Context *c);
 
 ESExpResult *func_opacity(ESExp *f, int argc, ESExpResult **argv, Context *c);
 
+ESExpResult *func_spawn_sync(ESExp *f, int argc, ESExpResult **argv, Context *c);
+ESExpResult *func_spawn_async(ESExp *f, int argc, ESExpResult **argv, Context *c);
+
 #endif /* _ACTIONS_H */
